from flask import flash
from loguru import logger
import helpers.custom_logging
from flask import Flask, request, render_template, Response
from flask_migrate import Migrate

import config
from helpers.file_utils import *


def create_app():
    import pwndoc_db_init
    from helpers.db_models import db
    from helpers.template_grouping import TemplateGrouping

    from api_process_findings import bp as process_findings_bp
    from api_templates import bp as templates_bp
    from api_debug import bp as debug_bp
    from api_root import bp as root_bp
    from api_pwndoc_audit import bp as pwndoc_bp
    import rq_dashboard

    logger.add("user_data/importer-logs/main.log", rotation="1 week", retention="60 days", compression="gz", encoding="utf8")

    pwndoc_db_init.InitialData()

    app = Flask(__name__)

    app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///user_data/importer-db/main.db'
    app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
    db.init_app(app)

    migrate = Migrate()

    # https://github.com/miguelgrinberg/Flask-Migrate/issues/61#issuecomment-208131722
    with app.app_context():
        if db.engine.url.drivername == 'sqlite':
            migrate.init_app(app, db, render_as_batch=True, compare_type=True)
        else:
            migrate.init_app(app, db)

    with app.app_context():
        db.create_all()

    app.config['UPLOAD_FOLDER'] = config.FLASK_UPLOAD_FOLDER
    app.config['MAX_CONTENT_LENGTH'] = 100 * 1000 * 1000
    app.config["JSONIFY_PRETTYPRINT_REGULAR"] = True
    app.config["SECRET_KEY"] = config.SECRET_KEY

    Path(app.config['UPLOAD_FOLDER']).mkdir(parents=True, exist_ok=True)

    app.register_blueprint(process_findings_bp, url_prefix='/findings')
    app.register_blueprint(templates_bp, url_prefix='/templates')
    app.register_blueprint(pwndoc_bp, url_prefix='/pwndoc_audit')
    app.register_blueprint(root_bp, url_prefix='/')

    if not config.RQ_DISABLED:
        app.config.from_object(rq_dashboard.default_settings)
        app.config["RQ_DASHBOARD_REDIS_URL"] = f'redis://{config.REDIS_HOST}:{config.REDIS_PORT}'
        app.register_blueprint(rq_dashboard.blueprint, url_prefix="/rq")
    else:
        @app.route("/rq")
        @app.route("/rq/")
        def rq_disabled():
            flash("Redis scheduling is disabled.", "info")
            return render_template('base.html')

    # Todo: possibly disable this or require additional password
    app.register_blueprint(debug_bp, url_prefix='/debug')

    TemplateGrouping.add_new_gids()

    @app.errorhandler(500)
    def internal_error(error):
        flash(f"Internal server error (more information is in server logs)", "danger")
        return render_template('base.html'), 500

    @app.errorhandler(404)
    def internal_error(error):
        flash(f"The requested URL doesn't exist.", "danger")
        return render_template('base.html'), 404

    return app


if __name__ == "__main__":
    app = create_app()
    app.run()  # todo: maybe change?
